UNIT Debugging;

INTERFACE
USES
  Globals;

PROCEDURE DisplayPieceLists;
PROCEDURE DisplayBoard;
PROCEDURE DisplayEval;
PROCEDURE WriteHistoryTable;

FUNCTION  Perft(Depth : Byte; x : Byte) : Word64;


IMPLEMENTATION
USES
  Crt,
  GenMoves,
  Zorbist,
  Move,
  Board,
  Utils,
  TT,
  LookUpTables;

{-----------------------------------------------------------------------------------------}

PROCEDURE DisplayPieceLists;
VAR
  i : Byte;
BEGIN
  WriteLn;
  WITH Data[Ply] DO
    BEGIN
      Write('White Pawns: ');
      FOR i := 0 TO 7 DO
        Write(WpawnList[i] : 4);
      WriteLn;
      Write('Black Pawns: ');
      FOR i := 0 TO 7 DO
        Write(BpawnList[i] : 4);
      WriteLn;

      Writeln('Rooks: ',   WpieceList[0, Wrook]   : 4, WpieceList[1, Wrook]   : 4, BpieceList[0, Brook]   : 4, BpieceList[1, Brook]   : 4);
      Writeln('Knights: ', WpieceList[0, Wnight]  : 4, WpieceList[1, Wnight]  : 4, BpieceList[0, Bnight]  : 4, BpieceList[1, Bnight]  : 4);
      Writeln('Bishops: ', WpieceList[0, Wbishop] : 4, WpieceList[1, Wbishop] : 4, BpieceList[0, Bbishop] : 4, BpieceList[1, Bbishop] : 4);
      Writeln('Queens: ',  WpieceList[0, Wqueen]  : 4, WpieceList[1, Wqueen]  : 4, BpieceList[0, Bqueen]  : 4, BpieceList[1, Bqueen]  : 4);
      Writeln('Kings: ',   WpieceList[0, Wking]   : 4, WpieceList[1, Wking]   : 4, BpieceList[0, Bking]   : 4, BpieceList[1, Bking]   : 4);
    END;
END;


PROCEDURE DisplayBoard;
VAR
  Rank,
  Fyle : Byte;
BEGIN
  FOR Rank := 7 DOWNTO 0 DO
    BEGIN
      FOR Fyle := 0 TO 7 DO
        Write( (Data[Ply].Board[Rank] SHR (4*Fyle)) AND %1111 : 4);
      WriteLn;
    END;
END;


PROCEDURE DisplayEval;
VAR
  i     : Byte;
  Score : Int16;
BEGIN
  ClrScr;
  DisplayBoard;
  DisplayPieceLists;

  Score := 0;
  WITH Data[Ply] DO
    BEGIN
      {-- first white and black pawns }
      FOR i := 0 TO 7 DO
        BEGIN
          GoToXY(50, i+1);
          Score := Score + PST_Pawn[WpawnList[i]];
          Write('Wp on ', WpawnList[i], '=', PST_Pawn[WpawnList[i]]);
          Score := Score - PST_Pawn[Flip(BpawnList[i])];
          Write('    Bp on ', BpawnList[i], '=', PST_Pawn[Flip(BpawnList[i])]);
        END;
      {-- now the pieces }
      FOR i := 0 TO 1 DO
        BEGIN
          GoToXY(50, 9+5*i);
          Score := Score + PST_Knight[WpieceList[i, Wnight]];
          Write('Wn on ', WpieceList[i, Wnight], '=', PST_Knight[WpieceList[i, Wnight]]);
          Score := Score - PST_Knight[Flip(BpieceList[i, Bnight])];
          Write('     Bn on ', BpieceList[i, Bnight], '=', PST_Knight[Flip(BpieceList[i, Bnight])]);

          GoToXY(50, 10+5*i);
          Score := Score + PST_bishop[WpieceList[i, Wbishop]];
          Write('Wb on ', WpieceList[i, Wbishop], '=', PST_bishop[WpieceList[i, Wbishop]]);
          Score := Score - PST_bishop[Flip(BpieceList[i, Bbishop])];
          Write('     Bb on ', BpieceList[i, Bbishop], '=', PST_bishop[Flip(BpieceList[i, Bbishop])]);

          GoToXY(50, 11+5*i);
          Score := Score + PST_rook[WpieceList[i, Wrook]];
          Write('Wr on ', WpieceList[i, Wrook], '=', PST_rook[WpieceList[i, Wrook]]);
          Score := Score - PST_rook[Flip(BpieceList[i, Brook])];
          Write('     Br on ', BpieceList[i, Brook], '=', PST_rook[Flip(BpieceList[i, Brook])]);

          GoToXY(50, 12+5*i);
          Score := Score + PST_queen[WpieceList[i, Wqueen]];
          Write('Wq on ', WpieceList[i, Wqueen], '=', PST_queen[WpieceList[i, Wqueen]]);
          Score := Score - PST_queen[Flip(BpieceList[i, Bqueen])];
          Write('     Bq on ', BpieceList[i, Bqueen], '=', PST_queen[Flip(BpieceList[i, Bqueen])]);

          GoToXY(50, 13);
          IF NOT EndGame THEN
            BEGIN
              Score := Score + PST_king_mg[WpieceList[0, Wking]];
              Write('Wk on ', WpieceList[0, Wking], '=', PST_king_mg[WpieceList[0, Wking]]);
              Score := Score - PST_king_mg[Flip(BpieceList[0, Bking])];
              Write('     Bk on ', BpieceList[0, Bking], '=', PST_king_mg[Flip(BpieceList[0, Bking])]);
            END
          ELSE
            BEGIN
              Score := Score + PST_king_eg[WpieceList[0, Wking]];
              Write('Wk on ', WpieceList[0, Wking], '=', PST_king_eg[WpieceList[0, Wking]]);
              Score := Score - PST_king_eg[Flip(BpieceList[0, Bking])];
              Write('     Bk on ', BpieceList[0, Bking], '=', PST_king_eg[Flip(BpieceList[0, Bking])]);
            END;
        END;
    END; {-- with }

  GoToXY(50, 18);
  Write('EndGame: ', IsEndGame);
  GoToXY(50, 19);
  Write('Material: ', CalcMaterial, '  Positional: ', Score);
  Readln;
  ClrScr;
END;


FUNCTION Perft(Depth : Byte; x : Byte) : Word64;
VAR
  i, j,
  MoveNum : Byte;
  Nodes,
  Total   : Word64;
  M       : MoveRec;
BEGIN
  Total   := 0;
  Nodes   := 0;
  j       := 0;

  IF Depth = 0 THEN
    Total := 1
  ELSE
    BEGIN
      MoveNum := GenMoveList('');

      FOR i := 1 TO MoveNum DO
        BEGIN
          M := GetAMove(MoveNum);

          Inc(Ply);
          Data[Ply] := Data[Ply - 1];

          MakeMove(M, False);

          IF NOT InCheck(-1*WhosMove) THEN
            BEGIN
              Nodes := Perft(Depth - 1, x);
              Total := Total + Nodes;
              Inc(j);

              IF Depth = x THEN
                BEGIN
                  Write(j : 2, ' ', SqToStr[M.FromSq]);
                  IF M.Capture THEN
                    Write('x');
                  Write(SqToStr[M.ToSq], Nodes : 10, '    ');
                  IF Odd(j) then
                    WriteLn;
                END;
            END;

          UnMakeMove;
        END;
    END;

  Perft := Total;
END;


PROCEDURE WriteHistoryTable;
CONST
  PieceTypeStr : ARRAY[1..14] OF String[7] = ('Wpawn', 'Wknight', 'Wbishop', 'Wrook', 'Wqueen', 'Wking', '', '',
                                              'Bpawn', 'Bknight', 'Bbishop', 'Brook', 'Bqueen', 'Bking');
VAR
  f : Text;
  P : PieceType;
  S : SqType;
BEGIN
  Assign(f, 'History.txt');
  ReWrite(f);

  FOR P := WPawn TO BKing DO
    FOR S := 0 TO 63 DO
      IF HistTab[P, S] <> 0 THEN
        WriteLn(f, PieceTypeStr[P], '->', S, HistTab[P, S] : 4);

  Close(f);
END;



BEGIN
END.
