UNIT Globals;

INTERFACE

 {-- the board is 64 Nibbles (32 bytes) and each nibble contains a piece or 0 for empty }
CONST
  NoPiece = -1;     {-- the piece lists use -1 as a non piece }
  Empty   =  0;     {-- the board uses 0 to show a vacant square }

  Wpawn   =  1;
  Wnight  =  2;
  Wbishop =  3;
  Wrook   =  4;
  Wqueen  =  5;
  Wking   =  6;
  {-- the black pieces has the msb of the nibble set }
  Bpawn   =  9;
  Bnight  = 10;
  Bbishop = 11;
  Brook   = 12;
  Bqueen  = 13;
  Bking   = 14;

  MaxPly   = 100;   {-- who am I kidding }
  MaxMoves = 100;  {-- per ply and in the whole game }

  Wmove    =  1;   {-- multiply by -1 to change sides }
  Bmove    = -1;

TYPE
  Int8      = ShortInt;
  Int16     = Integer;
  Int32     = LongInt;
  Word16    = Word;
  Word32    = LongWord;
  Word64    = Qword;

  PieceType = Int8;
  Str4      = String[4];

(*  {$ BITPACKING ON }*)
  BoardType = BITPACKED ARRAY[0..7] OF Word32; {-- each 32bit word contains 8 nibbles, array 0->7 gives the whole board }

  {-- the piece lists }
  WpieceListType = ARRAY[0..1, Wnight..Wking] OF Int8;  {-- lists where the pieces are on the board }
  BpieceListType = ARRAY[0..1, Bnight..Bking] OF Int8;
  PawnListType   = ARRAY[0..7] OF Int8;                 {-- list where the pawns are on the board }
  PieceValueType = ARRAY[1..14] OF Word16;              {-- contains the value of each piece in centipawns }

  PvType   = ARRAY[0..MaxPly, 0..MaxPly] OF Str4;       {- triangular array for the PV }

  ShortStr = String[10];
  MoveRec  = RECORD       {-- when generating moves, fill in the details }
               FromSq,
               ToSq      : Int8;
               MoveValue : Int16;
               Castle,
               Capture,
               Promotion : Boolean;
             END;

  DataType = BitPacked RECORD   {-- the data needed to represent the game at each ply }
                         Board      : BoardType;
                         WPieceList : WpieceListType;
                         BPieceList : BpieceListType;
                         WPawnList,
                         BPawnList  : PawnListType;
                         HashKey    : Word64;
                         EnPasSq    : Int8;
                         CastleStat : Byte; {-- bit0 = W0-0, bit1 = W0-0-0, bit 2 = B0-0, bit3 = B0-0-0 }
                       END;

CONST
  NoMove : MoveRec = (FromSq    : -1;
                      ToSq      : -1;
                      MoveValue : -999;
                      Castle    : False;
                      Capture   : False;
                      Promotion : False
                      );

  PieceValue : PieceValueType = (100, 320, 330, 500, 900, 10000, 0, 0,
                                 100, 320, 330, 500, 900, 10000);
  MaxMaterial = 4000; {-- dont count kings }

VAR
  Ply     : Byte = 0;
  Data    : ARRAY[0..MaxPly] OF DataType;
  HistTab : ARRAY[PieceType, 0..63] OF Word;  {-- scores for moves made during the search. Used for move scoring/sorting }

  CmdStr     : String = '';   {-- the user input }
  OpeningStr : String = '';   {-- built as we enter opening moves. Used to compare to library openings }

  MoveNum    : 0..MaxMoves = 0;
  MoveTime   : Word16 = 60;
  FixedDepth : Byte = 255;

  {-- boolean stuff }
  WhosMove   : Int8    = Wmove;  {-- 1 or -1, acts as a boolean }
  HumansMove : Boolean = True;
  EndGame    : Boolean = False;
  Debug      : Boolean = False;  {-- used only when debugging }
  InLibrary  : Boolean = True;


IMPLEMENTATION
{--------------------------------------------------------------------------------------------------------}

BEGIN
END.
