UNIT Move;


INTERFACE
USES
  Globals;


PROCEDURE MakeMove(TheMove : MoveRec; Qs : Boolean);
PROCEDURE UnMakeMove;

FUNCTION  IsLegalMove(TheMove : ShortStr) : Boolean;
PROCEDURE TakeBackMove;
PROCEDURE AddToTakeBackStack;
PROCEDURE InitTakeBackStack;


IMPLEMENTATION
USES
  Crt,
  Utils,
  Board,
  GenMoves,
  Zorbist,
  Opening,
  Search,
  TT;

{-----------------------------------------------------------------------------------------------}
VAR
  TakeBackStack : ARRAY[0..15] OF RECORD
                                   D : DataType;
                                   M : Byte;
                                   W : Int8;  {-- 1 or -1 }
                                 END;
   TBindex : Int8;    {-- always points to current position }



PROCEDURE InitTakeBackStack;
BEGIN
  FillChar(TakeBackStack, SizeOf(TakeBackStack), 0);
  TBindex := -1;

  AddToTakeBackStack;  {-- add the initial position }
END;


PROCEDURE AddToTakeBackStack;
VAR
  i : Byte;
BEGIN
  IF TBindex < 16 THEN
    Inc(TBindex);         {-- now points to current position }

  IF TBindex = 16 THEN    {-- shuffle the moves down to make space }
    BEGIN
      FOR i := 1 TO 14 DO
        TakeBackStack[i] := TakeBackStack[i + 1];
      TBindex := 15;
    END;

  {-- add the new state to the stack }
  WITH TakeBackStack[TBindex] DO
    BEGIN
      D := Data[Ply];
      M := MoveNum;
      W := WhosMove;
    END;
END;


PROCEDURE TakeBackMove;
BEGIN
  IF MoveNum > 0 THEN
    BEGIN
      IF TBindex > 0 THEN
        Dec(TBindex);

      WITH TakeBackStack[TBindex] DO
        BEGIN
          Data[Ply] := D;
          MoveNum   := M;
          WhosMove  := W;
        END;

      IF InLibrary THEN
        Delete(OpeningStr, Length(OpeningStr) - 3, 4);
    END;

  {-- zero tables }
  FillChar(PV, SizeOf(PV), 0);
  FillChar(HistTab, SizeOf(HistTab), 0);
  ZeroTTs;
END;


PROCEDURE MakeMove(TheMove : MoveRec; Qs : Boolean);
VAR
  FromPiece,
  ToPiece   : PieceType;
  M         : MoveRec;
BEGIN
  {-- count nodes only when a piece moves }
  IF NOT Qs THEN
    Inc(Nodes)
  ELSE
    Inc(Qnodes);

  FromPiece := WhatPiece(TheMove.FromSq);
  ToPiece   := WhatPiece(TheMove.ToSq);

  {-- pickup your piece }
  RemovePiece(TheMove.FromSq);

  {-- if the rook moves you can no longer castle on that side }
  IF FromPiece IN [Wrook, Brook] THEN
    BEGIN
      CASE TheMove.FromSq OF
         {-- white }
         0 : Data[Ply].CastleStat := Data[Ply].CastleStat AND %00001101;  { 0-0-0 }
         7 : Data[Ply].CastleStat := Data[Ply].CastleStat AND %00001110;  { 0-0 }
         {-- black }
        56 : Data[Ply].CastleStat := Data[Ply].CastleStat AND %00000111;  { 0-0-0 }
        63 : Data[Ply].CastleStat := Data[Ply].CastleStat AND %00001011;  { 0-0 }
      END;
    END;

  {-- check for castling }
  IF TheMove.Castle THEN
    BEGIN
      {-- first move the Rook across, by recursively calling MakeMove }
      FillChar(M, SizeOf(M), 0);

      IF WhosMove = Wmove THEN      {-- white moves }
        BEGIN
          Data[Ply].CastleStat := Data[Ply].CastleStat AND %00001100;  {-- castling now not allowed }

          IF TheMove.ToSq = 6 THEN  {-- castle short }
            BEGIN
              M.FromSq := 7;
              M.ToSq   := 5;
            END
          ELSE
            BEGIN                   {-- castle long }
              M.FromSq := 0;
              M.ToSq   := 3;
            END;
        END
      ELSE                          {-- black moves }
        BEGIN
          Data[Ply].CastleStat := Data[Ply].CastleStat AND %00000011;   {-- zero castling rights }

          IF TheMove.ToSq = 62 THEN
            BEGIN
              M.FromSq := 63;
              M.ToSq   := 61;
            END
          ELSE
            BEGIN
              M.FromSq := 56;
              M.ToSq   := 59;
            END;
        END;

      MakeMove(M, False);  {-- recursive call to move the castle }
      SwapSides;           {-- because makemove changed sides }
      Dec(MoveNum);        {-- cause the move isnt finished here }
    END;

  {-- remove the captured piece }
  IF TheMove.Capture THEN
    BEGIN
      IF ToPiece <> Empty THEN                   {-- it will be for Enpausant captures }
        RemovePiece(TheMove.ToSq);

      {-- now Enpausant capture }
      IF (TheMove.ToSq = Data[Ply].EnPasSq) AND (FromPiece IN [Wpawn, Bpawn]) THEN
        BEGIN
          RemovePiece(TheMove.ToSq - 8*WhosMove);
          Data[Ply].EnPasSq := -1;
        END;
    END;

  {-- handle a promotion }
  IF TheMove.Promotion THEN      {-- will fix under promotion later }
    IF WhosMove = 1 THEN
      FromPiece := Wqueen
    ELSE
      FromPiece := Bqueen;

    {-- put your piece down on the new square }
    PutPiece(FromPiece, TheMove.ToSq);

  {-- flag a potential enpausant move allowed }
  IF (FromPiece IN [Wpawn, Bpawn]) AND (Abs(TheMove.FromSq - TheMove.ToSq) = 16) THEN
    Data[Ply].EnPasSq := TheMove.ToSq - WhosMove*8
  ELSE
    Data[Ply].EnPasSq := -1;

  Inc(MoveNum);
  SwapSides;
END;


PROCEDURE UnMakeMove;
BEGIN
  Dec(MoveNum);
  Dec(Ply);
  SwapSides;
END;


FUNCTION IsLegalMove(TheMove : ShortStr) : Boolean;
{-- check if the humans move is legal by generating all moves and checking if }
{-- the move is one of the moves generated and does not leave you still in check }
VAR
  i,
  NumMoves : Byte;
  Move     : MoveRec;
BEGIN
  IsLegalMove := False;

  NumMoves := GenMoveList('');

  FOR i := 1 TO NumMoves DO
    BEGIN
      Move := GetAMove(NumMoves);
      IF TheMove = SquaresToStr(Move.FromSq, Move.ToSq) THEN {-- found the move made }
        BEGIN
          {-- now does this move leave you in check }
          Inc(Ply);
          Data[Ply] := Data[Ply - 1];

          MakeMove(Move, False);

          {-- white has moved and therefore WhosMove is Black, or vise versa so }
          IF NOT(InCheck(-1*WhosMove)) THEN
            IsLegalMove := True;

          UnMakeMove;
        END;  {-- if }
   END;     {-- for }
END;




BEGIN
END.
